<?php
// $Id$

/**
 * @file
 * OM Subthemer Admin
 *
 * @author: Daniel Honrade http://drupal.org/user/351112
 *
 * For users with admin access
 *  - admin can assign which content type to be used in OM Subtheming
 *
 */

/**
 * Configure Form
 * call back for admin/settings/om-subthemer menu
 */ 
function om_subthemer_admin($form, &$form_state) {
  
  drupal_set_title(t('OM Subthemer Settings'));
  
  $form = array();
  
  $form['om_subthemer'] = array(
    '#type' => 'fieldset',
    '#title' => t('Subtheme Types'),
    '#collapsible' => TRUE,
    '#collapsed' => FALSE,
    '#tree' => TRUE,
  );
  
  // scan and load all available base themes and skins
  $base_themes = om_available_base_themes();
  $skins = om_available_skins();
  
  // get all node types
  foreach (node_type_get_types() as $type => $info) {
    $configure[] = _om_subthemer_admin($info, $type, $base_themes, $skins);
  }

  $form['om_subthemer'] += $configure;  

  $form['submit'] = array( '#type' => 'submit', '#value' => t('Save Settings'), );

  return $form;
}

/**
 * Admin Configure form
 *
 */
function _om_subthemer_admin($info, $type, $base_themes, $skins) {
  $form['enabled'] = array(
    '#type' => 'checkbox',
    '#title' => t($info->name),
    '#default_value' => variable_get('om_subthemer_enabled_'. $type, 0),
  ); 
  
  $form['type'] = array(
    '#type' => 'hidden',
    '#default_value' => $type,
  );

  $form['base_theme'] = array(
    '#type' => 'select',
    '#title' => t('Base Theme'),
    '#default_value' => variable_get('om_subthemer_base_theme_'. $type, 'om'),
    '#options' => $base_themes,
  );
  
  $form['default_skin'] = array(
    '#type' => 'select',
    '#title' => t('Default Skin'),
    '#default_value' => variable_get('om_subthemer_skin_'. $type, 'starter'),
    '#options' => $skins,
  );
      
  return $form; 
}

/**
 * Submit
 * 
 */
function om_subthemer_admin_submit($form, $form_state) {
  
  $form_values = $form_state['values'];
  
  //dsm($form_values['om_subthemer']);
  foreach ($form_values['om_subthemer'] as $admin) {
    //dsm($admin);
    variable_set('om_subthemer_enabled_' . $admin['type'], $admin['enabled']);
    variable_set('om_subthemer_base_theme_' . $admin['type'], $admin['base_theme']);
    variable_set('om_subthemer_skin_'. $admin['type'], $admin['default_skin']);
  } 
  
  // for the new tab to appear
  cache_clear_all();
  menu_rebuild();  
}


/**
 * Implementation of theme_hook().
 *
 * View in table format
 *
 */
function theme_om_subthemer_admin($vars) {
  $form = $vars['form'];
  $rows = array();
  $output = '';

  foreach (element_children($form['om_subthemer']) as $key) {
    $row = array();
    // Strips labels
    //$form['om_subthemer'][$key]['enabled']['#title'] = '';
    $form['om_subthemer'][$key]['base_theme']['#title'] = '';
    $form['om_subthemer'][$key]['default_skin']['#title'] = '';

    $row[] = drupal_render($form['om_subthemer'][$key]['enabled']);
    $row[] = drupal_render($form['om_subthemer'][$key]['base_theme']);
    $row[] = drupal_render($form['om_subthemer'][$key]['default_skin']);
    $rows[] = $row;
  }
  
  // Header
  $header = array(t('Enabled'), t('Base Theme'), t('Default Skin'), );  

  $form['om_subthemer']['#children'] = theme('table', array('header' => $header, 'rows' => $rows));

  $output .= drupal_render_children($form);
  
  return $output;
}

